/*
   Copyright 2018-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/

namespace IRDB_SDK
{
	class TransformStepState_t; 
	class ThanosPlugin_t;

	using namespace std;
	class TransformStep_t
	{
		public:
			virtual        ~TransformStep_t(void) { } 

			// abstract methods for plugin to implement
			virtual string    getStepName(void) const = 0;

			virtual int       parseArgs(const vector<string> step_args) = 0;
			virtual int       executeStep()                             = 0;

		protected:
			TransformStep_t() { } // Thanos plugins can create these

			// helper methods for plugin to use 
			virtual File_t*              getMainFile()    final; 
			virtual FileIR_t*            getMainFileIR()  final; 
			virtual DatabaseID_t         getVariantID()   final; 
			virtual IRDBObjects_t*const  getIRDBObjects() final; 

		private:
			// private data can only be accessed by protected accessors.
			TransformStepState_t* m_state = nullptr;
			virtual void setState(TransformStepState_t* p_state) final;
			friend class ThanosPlugin_t;

	};

}


// Must also declare a GetTransformStep global function in your thanos-enabled .so file
// so that thanos can use it as a factory to create your transform class.  It must
// follow this signature:
//
// extern "C" std::shared_ptr<TransformStep_t> getTransformStep(void);    
// {
//         return shared_ptr<TransformStep_t>(new YourClassHere_t());
// }



