/*
   Copyright 2018-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/

#include <string>
#include <set>
#include <map>
#include <irdb-core>

namespace IRDB_SDK
{

	using namespace std;

	class Transform_t
	{

		public:
			Transform_t(FileIR_t * file); 
			virtual ~Transform_t() { } 

			Instruction_t* insertAssemblyBefore  (Instruction_t* p_before, const string &p_asm, Instruction_t* p_targ=nullptr);
			Instruction_t* insertAssemblyAfter   (Instruction_t* p_after , const string &p_asm, Instruction_t* p_targ=nullptr);
			Instruction_t* insertDataBitsBefore  (Instruction_t* p_before, const string &p_asm, Instruction_t* p_targ=nullptr);
			Instruction_t* insertDataBitsAfter   (Instruction_t* p_after , const string &p_asm, Instruction_t* p_targ=nullptr);

			Instruction_t* addNewDataBits        (const string& p_bits);
			Instruction_t* addNewAssembly        (const string& p_asm);

			void setInstructionAssembly(
					Instruction_t* instr, 
					const string& p_asm, 
					Instruction_t *p_new_fallthrough, 
					Instruction_t* p_new_target=nullptr);

			void setInstructionAssembly(Instruction_t* instr, const string& p_asm);


			FileIR_t* getFileIR();  
			const FileIR_t* getFileIR() const;  

		private:

			FileIR_t    *m_fileIR;
	};

}


